export default class Sandbox {
  constructor() {
    if (typeof document !== 'undefined') {
      this.frame = document.createElement('iframe');
      this.frame.style.display = 'none';
      this.frame.addEventListener('load', () => {
        this.frame.contentWindow.executeWindow = {};
        this.executeWindow = this.frame.contentWindow.executeWindow;
      });

      document.body.appendChild(this.frame);
    }
  }

  execute(code, blockNumber) {
    let promise = Promise.resolve();
    const exec = {
      blockNumber
    };

    try {
      exec.result = this.frame.contentWindow.eval(`with(executeWindow){${code}}`);
    } catch (error) {
      exec.error = error;
    }

    if (exec.result && exec.result.then && typeof exec.result.then === 'function') {
      promise = exec.result
        .then((value) => {
          exec.result = value;
        })
        .catch((error) => {
          exec.error = error;
        });
    }

    return promise
      .then(() => {
        this.executeWindow[`$${blockNumber}`] = exec.error || exec.result;

        return exec;
      });
  }

  dispose() {
    this.frame.parentNode.removeChild(this.frame);
  }
}
